aic = function(ts, tmm, model, lo, hi)
# 'lo' and 'hi' are bounds for the model's parameters.
# 'tmm' is needed for posterior means with which to initialize
# 'optim'.
   {f = model$choice.p
    params = names(formals(f))[-1]
    posterior.means = dcast(ss(tmm, !is.na(param)),
       s ~ param, value.var = "mean")
    row.names(posterior.means) = posterior.means$s
    ddply(ts, .(s), function(slice)
       {o = optim(
            as.numeric(posterior.means[char(slice$s[1]), params]),
            function(theta)
              {if (any(theta <= lo | theta >= hi))
                  NA
               else
                 {ps = pmax(1e-15, pmin(1 - 1e-15,
                      do.call(f, c(list(slice), theta))))
                  sum(log(ifelse(slice$choice == "ll", ps, 1 - ps)))}},
            control = list(fnscale = -1, maxit = 5000))
        if (o$convergence != 0)
           {dput(slice)
            print(o)
            stop(sprintf("convergence = %d: %s", o$convergence, if (is.null(o$message)) "" else o$message))}
        data.frame(aic = o$value - length(params))})}

aics.by.model = function(ts, fitall)
   {#ts = ss(ts, s %in% good.s & type == "test")
    bounds = list(
        expk.rho = list(lo = c(0, 0), hi = c(1, 1)),
        ghmk.rho = list(lo = c(0, 0, 0), hi = c(1, 1, 1)),
        diff.rho = list(lo = c(-1, 0), hi = c(1, 1)),
        sr.rho = list(lo = c(-1, 0), hi = c(1, 1)))
    do.call(rbind, lapply.names(models.by.name, function(mname)
         data.frame(model = mname,
             aic(ts, fitall[[mname]]$tmm, models.by.name[[mname]],
                 lo = bounds[[mname]]$lo,
                 hi = bounds[[mname]]$hi))))}

loglik.byhand = function(f, slice, theta)
      {if (F)#any(theta <= lo | theta >= hi))
          NA
       else
         {ps = pmax(1e-15, pmin(1 - 1e-15,
              do.call(f, c(list(slice), theta))))
          sum(log(ifelse(slice$choice == "ll", ps, 1 - ps)))}}

rmse.byhand = function(f, slice, theta)
      {if (F)#any(theta <= lo | theta >= hi))
          NA
       else
         {ps = do.call(f, c(list(slice), theta))
          sqrt(mean( ((slice$choice == "ll") - ps)^2))}}
