# --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# Subject-running notes and extra data
# --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

# For subjects run on 5 Nov 2012,
#   129.49.113.242 is the analysis computer in 204
#   129.49.112.169 is PsychA206L
# s0006 hit the blankness bug.
# For s0032, Firefox crashed before he could so much as
#   acknowledge the instructions, and I let him go.

bad.s = unique(qw(
    s0012,
      # Used the old 'sr' model.
    s0031,
      # Fell for two catch trials.
    s0025, s0044, s0100, s0066, s0145, s0148, s0157, s0178, s0233, s0277, s0333,
      # test.minutes < 4.
    s0106, s0157, s0191, s0252, s0258, s0269, s0288, s0304, s0305,
      # test.lls == 100.
    #s0031, s0104, s0107, s0112, s0147, s0151, s0178, s0237, s0283, s0277, s0302, s0321, s0344,
      # 95 < test.lls < 100.
    s0255))
      # Did a large part of the task earlier (as subject 224).

from.prescreens = data.frame(
  # The version of the task I used for these subjects
  # didn't ask for demographics.
    row.names = qw(s0009,  s0012,  s0017,  s0018,  s0019),
    gender = qw(   Female, Female, Female, Male,   Male),
    age = c(       18,     17,     18,     17,     21),
    hand = qw(     Righty, Righty, Righty, Righty, Righty))

# --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# Make sb (subjects) and ts (trials) 
# --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

vassign(.(sb, dlong, dwide, timing, tdiff), unpack.tversky(
     "builder.sqlite",
     include.incomplete = F,
     exclude = 328))
       # For s0328, an entirely mysterious bug stopped the
       # adaption at round 10.
mcmc.quits = with(
    dbGetQuery(
        dbConnect(dbDriver("SQLite"), dbname = "asj.sqlite"),
        "select subject, sum(quit_early) from MCMCDiagnostics group by subject"),
    `names<-`(`sum(quit_early)`, tversky.sid(subject)))

sb = with(dwide, transform(sb,
    test.minutes = (ss(timing, k == "test_choice.100")$received -
        ss(timing, k == "test_choice.3")$received)/60,
    where = ifelse(s %in% qw(s0006, s0008, s0009, s0012),
        "Psych A 204",
        ifelse(!is.na(experimenter),
        "Psych A 206 or 211",
        "MTurk")),
    gender = gender,
    age = as.numeric(age),
    hand = ifelse(handedness == "Left-handed", "Lefty", "Righty"),
    model = model,
    mcmc.quits = mcmc.quits[char(s)]))
sb[row.names(from.prescreens), names(from.prescreens)] = from.prescreens

good.s = setdiff(row.names(sb), bad.s)

ts = ordf(
    transform(ss(dlong, str_detect(k, fixed("adaptive_quartet"))),
        tn = as.numeric(str_extract(k, "\\d+$"))),
       # paste(s, "adaptive_choice.", str_extract(k, "\\d+$"), sep = "")
       #     %in% paste(dlong$s, dlong$k, sep = "")),
          # The second condition here ensures we only include
          # trials that the subject responded to.
    s,
    tn)
ts = cbind(
    ts$s,
    ts$tn,
    as.data.frame(t(sapply(
        strsplit(ss(ts, str_detect(k, fixed("quartet")))$v, " ", fixed = T),
        as.numeric))),
    factor(
        ordf(ss(dlong, str_detect(k, "adaptive_choice\\.\\d+$")),
            s,
            as.numeric(str_extract(k, "\\d+$")))$v,
        levels = qw(ss, ll)),
    logi2factor(
        ordf(ss(dlong, str_detect(k, "adaptive_choice\\.\\d+.permutation$")),
            s,
            as.numeric(str_extract(k, "\\d+$")))$v == "ss,ll",
        qw(left, right)))
names(ts) = qw(s, tn, ssr, ssd, llr, lld, choice, ll.side)
ts$type = factor("train", levels = qw(train, test, catch))

test.ts = with(list(x = strsplit(dwide$test_quartets, ";", fixed = T)),
    do.call(rbind, lapply(seq_along(x), function (i)
       data.frame(
           s = row.names(dwide)[i],
           tn = seq_along(x[[i]]),
           t(sapply(strsplit(x[[i]], ",", fixed = T), as.numeric))))))
test.ts$choice = factor(
    ordf(ss(dlong, str_detect(k, "test_choice\\.\\d+$")),
        s,
        as.numeric(str_extract(k, "\\d+$")))$v,
    levels = qw(ss, ll))
test.ts$ll.side = logi2factor(
    ordf(ss(dlong, str_detect(k, "test_choice\\.\\d+.permutation$")),
        s,
        as.numeric(str_extract(k, "\\d+$")))$v == "ss,ll",
    qw(left, right))
names(test.ts) = qw(s, tn, ssr, ssd, llr, lld, choice, ll.side)
test.ts = transform(test.ts, type = factor(
    ifelse(ssr >= llr, "catch", "test"),
    levels = qw(train, test, catch)))

ts = ordf(rbind(ts, test.ts), s, type, tn)
ts = cbind(ts[, 1:2], qn = NA, ts[, -(1:2)])
ts = transform(ts, qn = ifelse(type == "test",
    charmatch(paste(ssr, ssd, llr, lld), test.quartets),
    NA))

sb$test.lls = aggregate(
  # How many choices in the test set were LL.
    choice ~ s,
    ss(ts, type == "test"),
    function (v) sum(v == "ll"))$choice
sb$test.rights = daply(
  # How many choices in the test set were the right-hand choice.
    ss(ts, type == "test"),
    .(s),
    function (slice) sum(
        xor(slice$choice == "ll", slice$ll.side == "left")))
sb$unique.train = daply(
  # How many unique training trials the subject saw.
    ss(ts, type == "train"),
    .(s),
    function (slice) with(slice,
        length(unique(paste(ssr, ssd, llr, lld)))))

sb$good = row.names(sb) %in% good.s
