library(ggplot2)

ghmkf =  function(v30, scurve)
   {curve <- 10/(1 - scurve) - 10
    e <- 1/-curve
    a <- (1/(v30^curve) - 1)/30
    function(delay) (1 + a * delay)^e}

example.ghmk.curves = function(v30 = .5, xmax = 60, ymin = 0, n = 101)
   {scurves = c(.001, .1, .2, .3, .4, .5, .6, .7, .8, .9, .999)
    xx = seq(.01, xmax, len = n)
    d = do.call(rbind, lapply(scurves, function (scurve)
        data.frame(scurve = scurve, x = xx, y = ghmkf(v30, scurve)(xx))))
    d$scurve = ordered(d$scurve)
    qplot(c(0, xmax), c(ymin, 1), geom = "blank") +
        geom_line(aes(x, y, color = scurve), data = d) +
        scale_x_continuous(name = "", expand = c(0, 0)) +
        scale_y_continuous(name = "", expand = c(0, 0)) +
        coord_cartesian(xlim = c(0, xmax), ylim = c(ymin, 1))}

#labelpos = data.frame(
#    x = 15,
#    y = .78,
#    t = ".001")

example.ghmk.curves.paper = function(v30 = .5, xmax = 60, ymin = 0, n = 300)
   {scurves = c(.001, .1, .2, .3, .4, .5, .6, .7, .8, .9)
    xx = seq(.01, xmax, len = n)
    d = do.call(rbind, lapply(scurves, function (scurve)
        data.frame(scurve = scurve, x = xx, y = ghmkf(v30, scurve)(xx))))
    d$scurve = ordered(d$scurve)
    qplot(c(0, xmax), c(ymin, 1), geom = "blank") +
        geom_line(aes(x, y, linetype = scurve), data = d) +
#        geom_text(aes(x, y, label = t), data = labelpos) +
        scale_x_continuous(name = "Delay (days)", expand = c(0, 0), breaks = seq(0, xmax, 5)) +
        scale_y_continuous(name = "Discount factor", expand = c(0, 0), breaks = seq(ymin, 1, .1)) +
        coord_cartesian(xlim = c(0, xmax), ylim = c(ymin, 1)) +
        theme(legend.position = "none") +
        no.gridlines()}
