library(reshape2)
library(plyr)
library(stringr)
library(DBI)
library(RSQLite)
library(R.cache)
#library(rstan)
library(ggplot2)
library(parallel)
library(bbmle)
library(e1071)
library(randomForest)

sys.source(getOption("Kodi.R.path"), Kodi <- new.env())
for (name in Kodi$qw(
        qw, ordf, ss, ssq, evalq, vassign, df2nv, abbcor, zscore,
        mhilo, modlevels, revn,
        bernbeta.mean,
        boot.ranks, knn.autok, crossvalid,
        unpack.tversky, tversky.sid,
        lapply.names, bq, gimme,
        dodge, boxp, no.gridlines,
        daylight.autos
      ))
    assign(name, Kodi[[name]])
cached = Kodi$mk.cached(c("Kodi", "projects", "builder"))

options(fork_maybeparallel.sapply = F)

source("../adaptive-server/util.R", local = T)
source("../adaptive-server/quartets.R", local = T)
source("../adaptive-server/models.R", local = T)
source("../adaptive-server/adaptive.R", local = T)

models.by.name = list(
    expk.rho = model.expk.rho,
    ghmk.rho = model.ghmk.rho,
    diff.rho = model.diff.rho,
    sr.rho = model.sr.rho)

# --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# Quartets
# --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

std.ts = data.frame(
    ssr = c(10, 10, 22, 15, 24, 32, 25, 40, 50, 67, 47, 27, 83, 34, 10, 10, 22, 15, 24, 32, 25, 40, 50, 67, 47, 27, 83, 34, 10, 10, 22, 15, 24, 32, 25, 40, 50, 67, 47, 27, 83, 34),
    ssd = c(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30),
    llr = c(95, 89, 120, 64, 68, 93, 58, 67, 98, 88, 58, 29, 86, 35, 95, 89, 120, 64, 68, 93, 58, 67, 98, 88, 58, 29, 86, 35, 95, 89, 120, 64, 68, 93, 58, 67, 98, 88, 58, 29, 86, 35),
    lld = c(5, 7, 8, 10, 10, 20, 25, 25, 70, 35, 50, 35, 35, 43, 20, 22, 23, 25, 25, 35, 40, 40, 85, 50, 65, 50, 50, 58, 35, 37, 38, 40, 40, 50, 55, 55, 100, 65, 80, 65, 65, 73))

test.quartets = c("15 5 60 7", "45 0 125 1", "5 56 65 60.875", "5 2 60 14", "30 0 70 2", "10 3 50 7", "40 0 105 1", "5 5 35 14", "15 0 25 1", "95 0 140 1", "10 7 60 10", "15 0 80 7", "15 5 95 21", "25 10 100 30.4375", "25 0 105 10", "40 3 85 7", "75 0 140 2", "55 3 90 5", "20 0 90 10", "70 4 80 5", "15 2 75 14", "20 10 70 21", "90 4 125 7", "45 3 70 7", "35 0 80 4", "20 3 40 7", "80 5 160 14", "100 7 170 14", "25 0 50 4", "60 5 120 14", "80 0 115 5", "70 10 85 14", "70 3 120 10", "50 0 65 2", "10 4 85 91.3125", "40 3 110 21", "30 21 90 56", "95 0 120 5", "60 4 140 30.4375", "95 56 96 60.875", "20 0 45 10", "75 1 85 3", "40 5 110 60.875", "90 60.875 130 91.3125", "25 2 50 14", "55 30.4375 130 91.3125", "35 4 70 21", "60 10 90 21", "55 14 120 60.875", "80 0 95 5", "60 7 140 60.875", "10 14 55 121.75", "40 2 110 91.3125", "95 14 160 56", "95 3 100 7", "25 0 85 91.3125", "10 60.875 20 121.75", "70 3 105 30.4375", "80 14 150 91.3125", "60 3 115 60.875", "80 7 105 21", "40 91.3125 50 121.75", "95 2 155 60.875", "50 5 75 30.4375", "30 4 85 121.75", "25 56 35 91.3125", "90 30.4375 150 121.75", "85 56 110 121.75", "95 1 160 91.3125", "15 60.875 20 91.3125", "80 5 135 121.75", "90 4 135 60.875", "75 1 125 60.875", "40 0 80 91.3125", "90 7 100 14", "85 7 130 121.75", "10 2 25 91.3125", "20 0 30 21", "100 0 140 121.75", "100 21 115 60.875", "70 1 90 121.75", "25 30.4375 30 56", "25 7 30 21", "25 2 30 21", "30 3 31 5", "30 7 40 56", "85 0 86 3", "95 10 105 121.75", "80 3 85 30.4375", "70 10 80 56", "10 10 15 60.875", "95 10 100 91.3125", "20 21 25 121.75", "60 21 65 121.75", "85 4 90 60.875", "80 14 85 56", "80 1 85 121.75", "75 7 76 30.4375", "45 2 46 30.4375", "80 14 81 56")
  # These are the test quartets roughly in order of decreasing
  # probability of choosing LL. To get a new order for the current
  # dataset:
  # cat(gsub("\\s+", " ", perl = T, paste(collapse = " ", deparse(with(ordf(aggregate(choice ~ qn + ssr + ssd + llr + lld, ss(ts, type == "test"), function(v) mean(v == "ll")), choice, llr - ssr, -ssr), rev(paste(ssr, ssd, llr, lld)))))), "\n")

format.delay = function (d)
  {if (d == 0)
       'now'
   else if (d %% 7 == 0)
       sprintf('in %.0fw', d / 7)
   else if (d %% monthlen == 0)
       sprintf('in %.0fm', d / monthlen)
   else sprintf('in %.0fd', d)}

formatted.test.quartets = as.vector(sapply(test.quartets, function (q)
   {q = as.numeric(str_split(q, fixed(" "))[[1]])
    sprintf("$%.0f %s vs $%.0f %s",
        q[1], format.delay(q[2]), q[3], format.delay(q[4]))}))

scale_y_quartet = scale_y_continuous(
    name = "quartet", expand = c(0, 0),
    limits = c(1, 100),
    breaks = seq(2, 100, 2),
    labels = formatted.test.quartets[rep(c(T, F), 50)])

# --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# Suboordinate files
# --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

source("data.R", local = T)
source("simulation.R", local = T)
source("single-subject.R", local = T)
source("tmm.R", local = T)
source("fitall.R", local = T)
source("xvalid.R", local = T)
source("aic.R", local = T)
source("ghmk-ex.R", local = T)
source("misc.R", local = T)
daylight.autos("paper.org")
