# ------------------------------------------------------------
# Generating fake data
# ------------------------------------------------------------

simulate.adaption = function(procedure, model, decider)
   {ts = empty.ts
    x = procedure(ts, model)
    repeat
       {cat(sprintf(" [%d] ", nrow(ts)))
        if (x$final_trial)
            break
        ts = rbind(ts, decider(x$quartet))
        x = do.call(procedure, c(list(ts, model), x$state))}
    model$sample.posterior(ts)}

mkdecider = function(model, ...)
    function(quartet)
        gendata(model$choice.p, quartet, ...)

gendata = function(choice.p, ts, ...)
# Adds true.p and a simulated choice column to the data frame.
   {transform(
        cbind(ts, true.p = do.call(choice.p, c(
            list(ts), list(...)))),
        choice = logi2factor(rbinom(nrow(ts), 1, true.p), qw(ss, ll)))}

# ------------------------------------------------------------
# Analyzing fake data
# ------------------------------------------------------------

check.model.precision = function(ts.train, ts.test, model, ...)
   {ts.train = model$gendata(ts.train, ...)
    gd = model$gendata(ts.test, ...)
    p = model$predict.choices(ts.train, ts.test)
    with(ss(p, !is.na(trial)), list(
        p = p,
        stats = t(t(local({
            qs = quantile(hi - lo, c(.025, .25, .75, .975))
            c(qs[1], qs[2], meanwidth = mean(irng), qs[3], qs[4],
                coverage = mean(gd$true.p >= lo & gd$true.p <= hi))})))))}

compare.models = function(true.model, true.params, models)
   {d = true.model$gendata(std.ts, true.params)
    quartets = repr.ts[1:9,]
    qs = with(quartets, sprintf("$%d in %d d", llr, lld))
    qs = factor(qs, levels = qs)
    true = data.frame(
        q = qs,
        p = true.model$choice.p(quartets, true.params))
    preds = data.frame(
        q = qs,
        model = factor(rep(seq_along(models), each = length(qs)),
            labels = char(models)),
        do.call(rbind, lapply(eval.quoted(models),
            function (m) m$predict.choices(d, quartets))))
    list(true = true, preds = preds)}

compare.models.plot = function(true, preds)
    ggplot(data = preds) +
        geom_pointrange(
            aes(x = q, ymin = lo, y = mean, ymax = hi, color = model),
            position = position_dodge(width = .5, height = 0)) +
        geom_point(data = true,
            aes(x = q, y = p)) +
        scale_color_hue(l = 20, c = 200) +
        scale_y_continuous(limits = c(0, 1), expand = c(0, 0))

#bin.hdrs = function(ts, variables, hdr.f, mass = .95)
#    ddply(ts,
#        variables,
#        .parallel = T, .progress = "text",
#        function (slice)
#           {x = hdr.f(slice, mass)
#            data.frame(hdr.points = x$hdr, mode = x$mode)})
#
#plot.bin.hdrs = function(d)
#    ggplot(data = d) +
#    geom_point(
#        aes(x = s, y = hdr.points),
#        color = "black",
#        size = .75) +
#    geom_point(
#        aes(x = s, group = s, y = true),
#        color = "red") +
#    geom_point(
#        aes(x = s, group = s, y = mode),
#        color = "blue") +
#    scale_x_discrete(breaks = NULL) +
#    scale_y_continuous(expand = c(0, 0),
#        limits = c(.0001, 1),
#        breaks = c(.0001 * 4^(0:6), 1)) +
#    coord_trans(ytrans = log_trans())
