# ------------------------------------------------------------
# Fitting multiple subjects to one model
# ------------------------------------------------------------

builder.test.model.multisubjects = function(ts, model)
   {ts = ordf(ts, s, type, qn)
    tmm = test.model.multisubjects(ts, model)
    tmm[!is.na(tmm$trial), "actual"] = ss(ts, test)$choice
    c(list(tmm = tmm), tmm.correct(ts, tmm))}

test.model.multisubjects = function(ts, model)
    ddply(ts, .(s), .progress = "text", function(slice)
       {print(char(slice$s[1]))
        summarize.posterior(
            model$sample.posterior(ss(slice, train)),
            ss(slice, test),
            model$choice.p)})

# ------------------------------------------------------------
# Summarizing 'tmm' objects
# ------------------------------------------------------------

tmm.param.plot = function(tmm, param.name)
    ggplot(ss(tmm, param == param.name)) +
        geom_pointrange(aes(
            x = rank(mean, ties.method = "first"),
            y = mean, ymin = lo, ymax = hi)) +
        scale_x_continuous(name = "", breaks = NULL) +
        ylab(param.name)

tmms.param.plot = function(tmms, param.name)
# 'tmms' should be several tmm frames 'rbind'ed together with
# a factor 'group' to distinguish them.
   {d = ss(ss(tmms, param == param.name),
        table(s)[char(s)] == nlevels(group))
    d$rank = rank(df2nv(aggregate(mean ~ s, d, FUN = mean)),
        ties.method = "first")[char(d$s)]
    ggplot(d) +
        geom_rect(aes(
            x = rank, xmin = rank - .5, xmax = rank + .5,
            ymin = lo, ymax = hi,
            fill = group),
          stat = "identity",
          alpha = .5,
          position = "identity") +
        scale_fill_discrete(l = 30) +
        scale_x_continuous(name = "", breaks = NULL) +
        scale_y_continuous(expand = c(0, 0)) +
        ylab(param.name) +
        theme(
            legend.justification = "right",
            legend.position = c(.9, .1),
            legend.background = element_rect(fill = "gray"))}

scale_rho = scale_y_log10(
    breaks = signif(c(1, .0001 * 2^(0:12)), 2),
    minor_breaks = c(),
    limits = c(.0001, 1),
    expand = c(0, 0))

#tmms.param.plot = function(tmms, param.name)
#   {d = ss(ss(tmms, param == param.name),
#        table(s)[char(s)] == nlevels(group))
#    d$rank = rank(df2nv(aggregate(mean ~ s, d, FUN = mean)),
#        ties.method = "first")[char(d$s)]
#    ggplot(d) +
#        geom_pointrange(aes(
#            x = rank,
#            y = mean, ymin = lo, ymax = hi,
#            color = group),
#            position = position_dodge(width = .5)) +
#        scale_color_discrete(l = 20, guide = "none") +
#        scale_x_continuous(name = "", breaks = NULL) +
#        ylab(param.name)}

tmm.correct = function(ts, tmm)
   {d = ddply(tmm, .(s), function (slice)
       {ts = ss(ts, s == slice$s[1])
        probs = ss(slice, !is.na(trial))$mean
        lls = ss(ts, test)$choice == "ll"
        d = data.frame(
            n = length(probs),
            correct = sum(xor(probs < .5, lls)),
            mjity = max(sum(lls), length(lls) - sum(lls)),
            mc = ifelse(mean(lls) > .5, "ll", "ss"),
            imp = 0,
            same = if
                (sign(mean(lls) - .5) ==
                 sign(mean(ss(ts, train)$choice == "ll") - .5))
              "" else "!")
        d = transform(d, imp = correct - mjity)
        for (r in which(!is.na(slice$param)))
           {d = cbind(d, slice[r, "lo"], slice[r, "hi"])
            names(d)[ncol(d) - (1:0)] = paste0(
                slice[r, "param"], c(".lo", ".hi"))}
        d})
    pred = ss(tmm, !is.na(trial))$mean
    each.s = ordf(d, imp, correct, -mjity)
    row.names(each.s) = each.s$s
    each.s$s = NULL
    list(
        total = round(sum(d$correct)/sum(d$n), 3),
        each.s = each.s,
        misses = transform(
            ordf(
                ss(cbind(ss(ts, test), pred),
                    !xor(choice == "ll", pred < .5)),
                -abs(pred - (choice == "ll"))),
            pred = round(pred, 3)))}

tmm.param.means.cor = function(tmm)
   {params = levels(tmm$param)
    abbcor(method = "spearman", `colnames<-`(
        do.call(cbind, lapply(params, function(p) 
            ss(tmm, param == p)$mean)),
        params))}
