

ply.ts = function(ses, inner.f)
   {#set.seed(seed)
    x = daply(ss(ts, s %in% ses & type == "test"), .(s = droplevels(s)), function(slice)
        {cat(char(slice$s[1]), " ")
         inner.f(slice[qw(ssr, ssd, llr, lld)], slice$choice)})
    message("")
    x}

# * Learners

prob2choice = function(probs) ifelse(probs >= .5, "ll", "ss")

fallback = function(limit, f) function(train.iv, train.dv, test.iv)
# If the training set is dominated by one choice or the other,
# just constantly predict that one choice.
   {tab = table(train.dv)
    if (any(tab <= limit))
        return(rep(names(tab)[which.max(tab)], nrow(test.iv)))
    f(train.iv, train.dv, test.iv)}

lf.mle = function(start, choice.p) fallback(length(start), function(train.iv, train.dv, test.iv)
# A helper function for learners that are fit by MLE.
   {minuslogl = eval(bq(
        function()
          {p = pmin(1 - 1e-12, pmax(1e-12, choice.p(train.iv, .(X))))
           -sum(log(ifelse(train.dv == "ll", p, 1 - p)))},
        list(X = lapply(names(formals(choice.p)[-1]), as.name))))
    formals(minuslogl) = formals(choice.p)[-1]
    rv = (function()
       {for (IGNORED in 1:3)
            tryCatch(
                return(mle2(
                    minuslogl = minuslogl,
                    start = lapply(start, function(x) rnorm(1, x, .05)),
                    method = "Nelder-Mead")),
                 error = function(e)
                     message("Got err: ", conditionMessage(e)))
        stop("failed after 3 errors")})()
    prob2choice(do.call(choice.p, c(list(test.iv), as.list(rv@coef))))})

learners = list(

# ** Majority model

    majority = list(pred = function(train.iv, train.dv, test.iv)
       {v = mean(train.dv == "ll")
        pred = (if (v < .5) "ss"
            else if (v > .5) "ll"
            else sample(c("ss", "ll"), 1))
        rep(pred, NROW(test.iv))}),

#do.xvalid.majority = function(ses, wrap.f = identity) ply.ts(ses, daply, function(iv, y)
#   {pred = crossvalid(rep(1, length(y)), y, nfold = 10, f = wrap.f(pf.majority))
#    sum(pred == y)})

# ** Nearest neighbors

# A very simple but still credible model.

    knn = list(rescale = T, pred = function(train.iv, train.dv, test.iv)
        knn.autok(train.iv, test.iv, train.dv)),

# ** Original models with MLE

# We use the original models (Exp, GH, Diff, and BT, as well as a
# hyperbolic model) without Bayesian methods. Instead, we get
# MLEs of the parameters and compute probabilities of choosing LL
# from these point estimates.

    exp.mle = list(pred = lf.mle(
        start = list(k = .01, ln.rho = log(.1)),
        choice.p = function(ts, k, ln.rho)
           {ssv = ts$ssr * exp(-k * ts$ssd)
            llv = ts$llr * exp(-k * ts$lld)
            ilogit(exp(ln.rho) * (llv - ssv))})),
    hyp.mle = list(pred = lf.mle(
        start = list(k = .01, ln.rho = log(.1)),
        choice.p = function(ts, k, ln.rho)         
           {ssv = ts$ssr / (1 + k * ts$ssd)
            llv = ts$llr / (1 + k * ts$lld)
            ilogit(exp(ln.rho) * (llv - ssv))})),
    ghm.mle = list(pred = lf.mle(
        start = list(ln.alpha = log(1), ln.beta = log(1), ln.rho = (.1)),
        choice.p = function(ts, ln.alpha, ln.beta, ln.rho)
           {alpha = exp(ln.alpha)
            beta = exp(ln.beta)
            ssv = ts$ssr * (1 + alpha * ts$ssd)^(-beta / alpha)
            llv = ts$llr * (1 + alpha * ts$lld)^(-beta / alpha)
            ilogit(exp(ln.rho) * (llv - ssv))})),
    sr.mle = list(pred = lf.mle(
        start = list(ln.gamma = log(1), ln.tau = log(1)),
        choice.p = function(ts, ln.gamma, ln.tau)
            {gamma = exp(ln.gamma)
             tau = exp(ln.tau)
             ilogit(
                (log(1 + gamma * ts$llr) - log(1 + gamma * ts$ssr))/gamma - 
                (log(1 + tau * ts$lld) - log(1 + tau * ts$ssd))/tau)})),

#    diff = list(
#        start = list(dr = -3, ln.rho = log(.1)),
#        f = function(ts, dr, ln.rho)
#            ilogit(exp(ln.rho) * ((ts$llr - ts$ssr) + dr * (ts$lld - ts$ssd)))),

    # diff is a GLM, so we might as well use 'glm' instead of
    # 'mle' or 'mle2'.

    diff.glm = list(pred = function(train.iv, train.dv, test.iv)
       {model = glm(choice ~ I(llr - ssr) + I(lld - ssd) + 0,
           data = cbind(train.iv, choice = train.dv),
           family = binomial(link = "logit"))
        prob2choice(predict(model, newdata = test.iv, type = "response"))}),

# ** Full logistic regression

# This model has five parameters: one for each quartet component
# and an intercept.

    full_logit = list(pred = function(train.iv, train.dv, test.iv)
       {model = glm(choice ~ ssr + ssd + llr + lld,
           data = cbind(train.iv, choice = train.dv),
           family = binomial(link = "logit"))
        prob2choice(predict(model, newdata = test.iv, type = "response"))}),

# ** SVM

    svm = list(rescale = T, pred = fallback(2, function(train.iv, train.dv, test.iv)
       tryCatch(
            char(predict(
                best.svm(x = train.iv, y = train.dv,
                    scale = F, type = "C-classification",
                    gamma = 2^seq(-25, 5, len = 5),
                    cost = 2^seq(-5, 25, len = 5),
                    tunecontrol = tune.control(cross = 5),
                    cachesize = 200, tolerance = .05),
                test.iv)),
            error = function(e)
              # Probably a "model is empty" error. Fall back on
              # the majority model.
                rep((if (mean(train.dv == "ll") > 1/2) "ll" else "ss"),
                    nrow(test.iv))))),

# ** Random forests

    rf = list(pred = fallback(2, function(train.iv, train.dv, test.iv)
       {# Choose the mtry with the best out-of-bag error
        # rate on the training set.
        forests = lapply(1:4, function(mtry)
            randomForest(x = train.iv, y = train.dv, mtry = mtry))
        forest = forests[[which.min(sapply(forests,
            function(rf) rf$err.rate[rf$ntree, "OOB"]))]]
        char(predict(forest, test.iv))})))

# ** Extra learner: saturated logit

# Like full_logit, but with all the interaction terms. All of them.

extra.learner.saturated_logit = list(pred = function(train.iv, train.dv, test.iv)
   {model = glm(choice ~ ssr * ssd * llr * lld,
       data = cbind(train.iv, choice = train.dv),
       family = binomial(link = "logit"))
    prob2choice(predict(model, newdata = test.iv, type = "response"))})

# * xvalid setup

corrupt.training.data.mxw = function(train.size, pred.f) function(train.iv, train.dv, test.iv)
   {# Set all but train.size trials in train.dv to random values.
    n = length(train.dv) - train.size
    train.dv[sample.int(length(train.dv), n)] =
        ifelse(rbinom(n, 1, .5), "ss", "ll")
    pred.f(train.iv, train.dv, test.iv)}

shrink.training.data.mxw = function(train.size, pred.f) function(train.iv, train.dv, test.iv)
   {# Select n trials from the training set and use only those.
    use = sample.int(length(train.dv), train.size)
    pred.f(train.iv[use,], train.dv[use], test.iv)}

metaparam.xvalid = function(wrap, ses, train.sizes, lnames = names(learners), cv.folds = 10)
# Do a round of cross-validation on the subjects 'ses' with
# all learners for each value of the metaparameter 'train.sizes',
# feeding the 'train.size' and learner into 'wrap'.
    do.call(rbind, lapply(train.sizes, function(train.size)
        data.frame(train.size, do.call(rbind, lapply(lnames, function(lname)
           {message("train.size ", train.size, ", learner ", lname)
            correct = ply.ts(ses, function(iv, y)
               {if (!is.null(learners[[lname]]$rescale))
                    for (col in seq_len(ncol(iv)))
                        iv[,col] = zscore(iv[,col])
                pred = crossvalid(iv, y, nfold = cv.folds, f =
                    wrap(train.size, learners[[lname]]$pred))
                sum(char(pred) == char(y))})
            data.frame(learner = lname, s = ses, correct)})))))

individual.xvalid = function(ses, lnames = names(learners), extra.learners = list(), cv.folds = 10)
# Do a round of cross-validation on the subjects 'ses' with
# all learners, and return a data frame with individual choices
# and predictions.
   {learners = c(learners, extra.learners)
    do.call(rbind, lapply(lnames, function(lname)
       {message("learner ", lname)
        d = do.call(rbind, dlply(ss(ts, s %in% ses & type == "test"), .(s = droplevels(s)), function(slice)
           {cat(char(slice$s[1]), " ")
            iv = slice[qw(ssr, ssd, llr, lld)]
            if (!is.null(learners[[lname]]$rescale))
                for (col in seq_len(ncol(iv)))
                    iv[,col] = zscore(iv[,col])
            pred = crossvalid(iv, slice$choice, nfold = cv.folds, f =
                learners[[lname]]$pred)
            cbind(slice[c("s", "qn", "choice")], pred)}))
        message("")
        row.names(d) = NULL
        data.frame(learner = lname, d)}))}

## Attempts at direct-labeling nonsense for correct.by.train.size.plot.
#library(directlabels)
#   if (direct)
##       direct.label(p,
##           list(polygon.method("last.points", 0.1, "left", "right"),
##            rot = c(30, 180),
##           col = "white"))
#       direct.label(p, "last.qp")
#   else
#       p}

correct.by.train.size.plot = function(bump, d)
    ggplot(transform(d, learner = factor(learner, levels = rev(levels(learner)))),
            if (bump)
                aes(train.size, correct + (num(learner)/10 - .5))
            else
                aes(train.size, correct)) +
        geom_line(aes(color = learner), size = 1.5) +
        scale_color_brewer(name = "Model", palette = "Paired") +
        geom_point() +
        scale_x_continuous(breaks = seq(0, 90, 10)) +
        scale_y_continuous(breaks = seq(0, 90, 2))

correct.by.train.size.plot.bw = function(bump, d)
   ggplot(transform(d, learner = factor(learner, levels = rev(levels(learner)))),
            if (bump)
                aes(train.size, correct + (num(learner)/10 - .5))
            else
                aes(train.size, correct)) +
        geom_line(aes(linetype = learner)) +
        scale_linetype(name = "Model") +
        geom_point() +
        scale_x_continuous(breaks = seq(0, 90, 10)) +
        scale_y_continuous(breaks = seq(0, 90, 2)) +
        no.gridlines() +
        theme(
            legend.key.width = unit(.75, "inch"),
            axis.title.y = element_text(vjust = 1.5))

#corrupting.xvalid.new = function(ses, corrupt.ns, cv.folds = 10)
#    do.call(rbind, lapply(corrupt.ns, function(corrupt.n)
#        data.frame(corrupt.n, do.call(rbind, lapply(names(learners), function(lname)
#           {message("corrupt.n ", corrupt.n, ", learner ", lname)
#            correct = ply.ts(ses, function(iv, y)
#               {if (!is.null(learners[[lname]]$rescale))
#                    for (col in seq_len(ncol(iv)))
#                        iv[,col] = zscore(iv[,col])
#                pred = crossvalid(iv, y, nfold = cv.folds, f =
#                    (corrupt.training.data(corrupt.n))(learners[[lname]]$pred))
#                sum(char(pred) == char(y))})
#            data.frame(learner = lname, s = ses, correct)})))))

#noop.prep.f = function(iv, y)
#    list(iv, y)
#
#general.xvalid = function(ses, ply.f, prep.f = noop.prep.f, inner.f)
#   {set.seed(50)
#    ply.f(ss(ts, s %in% ses & type == "test"), .(droplevels(s)), .progress = "text", function(slice)
#       {l = prep.f(slice[qw(ssr, ssd, llr, lld)], slice$choice)
#        crossvalid(l[[1]], l[[2]], nfold = 10,
#            
#        crossvalid(
#        inner.f(, slice$choice))}

